package com.me.brickbuster.state;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer.ShapeType;
import com.badlogic.gdx.math.MathUtils;
import com.me.brickbuster.BrickBuster;
import com.me.brickbuster.entity.Ball;
import com.me.brickbuster.entity.Brick;
import com.me.brickbuster.entity.Paddle;
import com.me.brickbuster.entity.powerup.*;

import java.util.*;

public class PlayState extends State {

	public static final int SHIELD_HEIGHT = 5;
	public static final float POWERUP_CHANCE = 0.15f;

	public static final Map<Class<? extends PowerUp>, Integer> powerUpWeights;
	private static final int weightSum;

	public List<PowerUp> powerUps;
	public Paddle paddle;
	public List<Ball> balls;
	public List<Brick> bricks;

	private int shieldCount = 0;

	private float updateTime = 0f;

	public PlayState(BrickBuster game) {
		super(game);
	}

	@Override
	public void setup() {
		powerUps = new ArrayList<PowerUp>();
		paddle = new Paddle(this);

		bricks = new ArrayList<Brick>();
		for (int col = 0; col < 13; col++) {
			for (int row = 0; row < 7; row++) {
				int x = 15 + (col * (Brick.BLOCK_WIDTH + 10));
				int y = 15 + Brick.BLOCK_HEIGHT + (row * (Brick.BLOCK_HEIGHT + 10));
				Class<? extends PowerUp> powerUpType = null;
				if (MathUtils.randomBoolean(POWERUP_CHANCE)) {
					powerUpType = getWeightedPowerUp();
				}
				bricks.add(new Brick(this, powerUpType, x, BrickBuster.HEIGHT - y));
			}
		}

		balls = new ArrayList<Ball>();
		balls.add(new Ball(this));
	}

	@Override
	public void render() {
		long start = System.nanoTime();
		for (Brick block : bricks) {
			block.render(game.sr);
		}
		for (PowerUp powerUp : powerUps) {
			powerUp.render(game.sr);
		}
		for (Ball ball : balls) {
			ball.render(game.sr);
		}
		paddle.render(game.sr);

		if (getShieldCount() > 0) {
			game.sr.begin(ShapeType.Filled);
			game.sr.setColor(Color.SALMON);
			game.sr.rect(0, 0, BrickBuster.WIDTH, getShieldCount() * SHIELD_HEIGHT);
			game.sr.end();
		}
		long renderTime = System.nanoTime() - start;

		game.sb.begin();
		game.font.setColor(Color.GRAY);
		game.font.draw(game.sb, String.format("FPS: %d Update: %.2f ms Render: %.2f ms",
				Gdx.graphics.getFramesPerSecond(), updateTime/1000000f, renderTime/1000000f), 0, 13);
		game.sb.end();
	}

	@Override
	public void update(float dt) {
		long start = System.nanoTime();
		paddle.update(dt);

		for (Iterator<Ball> it = balls.iterator(); it.hasNext();) {
			Ball ball = it.next();
			ball.update(dt);
			if (ball.isDead()) {
				it.remove();
			}
		}
		if (balls.isEmpty()) {
			ballReset();
		}

		for (Iterator<PowerUp> it = powerUps.iterator(); it.hasNext();) {
			PowerUp powerUp = it.next();
			powerUp.update(dt);
			if(powerUp.isCaught()) {
				it.remove();
			}
		}

		if (bricks.isEmpty()) {
			// TODO: Fix this - go to an after-game menu
			//create();
		}
		updateTime = System.nanoTime() - start;
	}

	public int getShieldCount() {
		return shieldCount;
	}

	public void addShield() {
		shieldCount++;
		paddle.setY(paddle.getY() + SHIELD_HEIGHT);
	}

	public void removeShield() {
		shieldCount--;
		paddle.setY(paddle.getY() - SHIELD_HEIGHT);
	}

	public void ballReset() {
		Ball ball = new Ball(this);
		ball.setX(paddle.getX());
		ball.setY(Paddle.PADDLE_Y + Paddle.PADDLE_HEIGHT + Ball.RADIUS);
		balls.add(ball);
		paddle.setSticky(false);
		paddle.setWidth(Paddle.DEFAULT_WIDTH);
	}

	private static final Class<? extends PowerUp> getWeightedPowerUp() {
		int remaining = MathUtils.random(weightSum);
		for (Map.Entry<Class<? extends PowerUp>, Integer> entry : PlayState.powerUpWeights.entrySet()) {
			remaining -= entry.getValue();
			if (remaining < 0) {
				return entry.getKey();
			}
		}
		return null;
	}

	static {
		Map<Class<? extends PowerUp>, Integer> tmp = new HashMap<Class<? extends PowerUp>, Integer>();

		// Assign PowerUp weights here
		tmp.put(GluePowerUp.class, 30);
		tmp.put(LongerPaddlePowerUp.class, 40);
		tmp.put(MultiBallPowerUp.class, 20);
		tmp.put(ShieldPowerUp.class, 40);
		powerUpWeights = Collections.unmodifiableMap(tmp);

		int sum = 0;
		for (int x : PlayState.powerUpWeights.values()) {
			sum += x;
		}
		weightSum = sum;
	}

}
